# coding: utf-8

import datetime
import json

def datetime_handler(x):
    if isinstance(x, datetime.datetime):
        return x.isoformat()
    return x

def json_dumps(data):
    return json.dumps(data, default=datetime_handler, indent=4)




# coding: utf-8

from ecs_manager.functions import set_variables, merge_environ


def test_set_variables():
    d = [{
        'int': 123,
        'str': '{{env}}',
        'dict': {'nested_str': '{{ env }}'},
        'list': ['{{ env }}', {'hoge': '{{ hoge }}'}]
    }, ]
    variables = {'hoge': 'hogehoge', 'env': 'test'}
    set_variables(d, variables)
    assert d == [{
        'int': 123,
        'str': 'test',
        'dict': {'nested_str': 'test'},
        'list': ['test', {'hoge': 'hogehoge'}]
    }, ]


def test_merge_environ():
    definition = [
        {
            'environment': [
                {'name': 'hoge', 'value': 'hoge'},
                {'name': 'fuga', 'value': 'fuga'},
            ]
        }, {
        }
    ]

    merge_environ(definition, {'hoge': 'hogehoge', 'piyo': 'piyopiyo'})

    assert definition[0]['environment'][0] == {'name': 'fuga', 'value': 'fuga'}
    assert definition[0]['environment'][1] == {'name': 'hoge', 'value': 'hogehoge'}
    assert definition[0]['environment'][2] == {'name': 'piyo', 'value': 'piyopiyo'}
    assert definition[1]['environment'][0] == {'name': 'hoge', 'value': 'hogehoge'}
    assert definition[1]['environment'][1] == {'name': 'piyo', 'value': 'piyopiyo'}




# coding: utf-8

import re

def set_variables(definition, variables):
    def traverse(obj):
        if isinstance(obj, str):
            for k, v in variables.items():
                obj = re.sub(r'{{\s?' + k + r'\s?}}', v, obj)
            return obj
        if isinstance(obj, dict):
            for key, value in obj.items():
                obj[key] = traverse(value)
        if isinstance(obj, list):
            obj = [traverse(o) for o in obj]
        return obj
    traverse(definition)
    return definition


def merge_environ(container_definition, env_override):
    for cd in container_definition:
        env = {}
        for e in cd.get('environment', []):
            env[e['name']] = e['value']
        for key, val in env_override.items():
            env[key] = val
        cd['environment'] = [{'name': key, 'value': env[key]} for key in sorted(env.keys())]


def check_task_status(client, name, container_definition):
    saved_task = client.list_task_definitions(familyPrefix=name, sort='DESC')
    if not saved_task['taskDefinitionArns']:
        return {'changed': True, 'arn': None}
    latest_task_arn = saved_task['taskDefinitionArns'][0]
    saved_definition = client.describe_task_definition(taskDefinition=latest_task_arn)['taskDefinition']['containerDefinitions']
    changed = regularization_task(saved_definition) != regularization_task(container_definition)
    return {'changed': changed, 'arn': latest_task_arn}


def regularization_task(task):
    result = []
    for container in task:
        result.append({k: v for k, v in container.items() if v != []})
    return result





# coding: utf-8

import os
import sys
from setuptools import setup, find_packages
from pip.req import parse_requirements

setup(
    name='ecs_manager',
    version='0.1.2.4',
    url='https://github.com/pistatium/ecs_service_manager',
    author='pistatium',
    description='Manage ecs service',
    packages=find_packages(),
    install_requires=[
        'boto3',
        'click'
    ],
    entry_points={
        'console_scripts': [
            'ecs_manager=ecs_manager.cmd:main'
        ]
    },
)




# coding: utf-8

import json
import os.path

import click
import boto3
import botocore

from ecs_manager.utils.json_util import json_dumps
from ecs_manager.functions import set_variables, merge_environ, check_task_status

client = boto3.client('ecs')


def json_validator(context, param, value):
    if value is None:
        raise click.BadParameter('{} definition is required'.format(param.name))
    if value.startswith('@'):
        path = value[1:]
        if not os.path.isfile(path):
            raise click.BadParameter('{}: File path `{}` is not found'.format(param.name, path))
        with open(path, 'r') as f:
            value = f.read()
    try:
        return json.loads(value)
    except json.decoder.JSONDecodeError:
        raise click.BadParameter('{} definition is invalid json format'.format(param.name))


@click.group()
def cmd():
    pass


@cmd.command()
@click.argument('name')
@click.option('--cluster', '-c', required=True)
@click.option('--task_container_definition', '-t', callback=json_validator)
@click.option('--service_definition', '-s', callback=json_validator)
@click.option('--environment', '-e', callback=json_validator, default='{}')
@click.option('--variables', '-v', callback=json_validator, default='{}')
def deploy_service(name, cluster, task_container_definition, service_definition, environment, variables):
    variables.update({
        'name': name,
        'cluster': cluster,
    })
    merge_environ(task_container_definition, environment)
    set_variables(task_container_definition, variables)

    status = check_task_status(client, name, task_container_definition)
    
    if status['changed']:
        click.echo(status)
        try:
            res = client.register_task_definition(family=name, containerDefinitions=task_container_definition)
        except botocore.errorfactory.ClientException:
            click.echo(repr(sys.exc_info()[1]))
            click.echo(task_container_definition)
            return
        task_arn = res['taskDefinition']['taskDefinitionArn']
        click.echo(task_arn)
    else:
        click.echo('Task definition is not changed')
        task_arn = status['arn']

    res = client.describe_services(cluster=cluster, services=[name, ])
    if not res['services']:
        res = client.create_service(cluster=cluster, serviceName=name, taskDefinition=task_arn, **service_definition)
        click.echo('Created service')
        click.echo(json_dumps(res))
        return
    res = client.update_service(cluster=cluster, service=name, taskDefinition=task_arn, **service_definition)
    click.echo('Updated service')
    click.echo(json_dumps(res))


def main():
    cmd()


if __name__ == '__main__':
    main()
